#ifndef PORT_H
#define PORT_H

#include <htc.h>
#include "ES_Timers.h"

#define EnableInterrupts {GIE = 1;}
#define DisableInterrupts {GIE = 0;}

// these macros provide the wrappers for critical regions, where ints will be off
// but the state of the interrupt enable prior to entry will be restored.

/* This macro saves INTCON register and disables global interrupts. */
#define EnterCritical(__temp)  do{ __temp = INTCON; GIE = 0;}while(0)
/* This macro restores INTCON register saved in EnterCritical(). */
#define ExitCritical(__temp)  {INTCON = __temp;}


/* Rate constants for programming the SysTick Period to generate tick interrupts.
   These assume an 40MHz configuration, they are the values to be used to program
   the SysTick Reload Value (STRELOAD) register. STRELOAD is 24-bits wide and so
   the highest value is 0xFFFFFF (16,777,216) which equates to
   16777216*1000/40000000 = 419.4 mS.
   They are all listed as -1 because the actual cycle time includes 1 cycle to 
   reset from the max count back to zero, so, for example, to achieve a 4000 
   count cycle time, you load the register with 4000-1
 */
typedef enum {	ES_Timer_RATE_OFF  	=   (0),
				ES_Timer_RATE_100uS = 4000-1,
				ES_Timer_RATE_500uS = 20000-1,
				ES_Timer_RATE_1mS	= 40000-1,
				ES_Timer_RATE_2mS	= 80000-1,
				ES_Timer_RATE_4mS	= 160000-1,
				ES_Timer_RATE_5mS	= 200000-1,
				ES_Timer_RATE_8mS	= 320000-1,
				ES_Timer_RATE_10mS	= 400000-1,
				ES_Timer_RATE_16mS	= 640000-1,
				ES_Timer_RATE_32mS	= 1280000-1
} TimerRate_t;


// Prottypes for hardware specific functions

unsigned char IsTimerPending( void );
void ES_Timer_Init(TimerRate_t Rate);
void IncrCounter( void );

#endif
