/*
 * File:   Blink.c
 * Author: Michael Lin
 *
 * Created on May 7, 2016, 11:26 AM
 */

#include <xc.h>
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "PIC16F1788.h"
#include "Blink.h"
#include <stdio.h>


static uint8_t MyPriority;
static BlinkState_t CurrentState;
/****************************************************************************
 Function
     InitBlink

 Parameters
     uint8_t : the priorty of this service

 Returns
     boolean, False if error in initialization, True otherwise

 Description
     Saves away the priority,  and starts
     the top level state machine
 Notes

 Author
     Michael Lin
****************************************************************************/
bool InitBlink ( uint8_t Priority )
{
  MyPriority = Priority;
  
  // Put us into the Initial PseudoState
  CurrentState = InitializeBlink;
  ES_Event ThisEvent;
  ThisEvent.EventType = ES_INIT;
  PostBlink(ThisEvent);
  return true;
}


/****************************************************************************
 Function
     PostBlink

 Parameters
     ES_Event ThisEvent , the event to post to the queue

 Returns
     boolean False if the post operation failed, True otherwise

 Description
     Posts an event to this state machine's queue
 Notes

 Author
     Michael Lin
****************************************************************************/
bool PostBlink( ES_Event ThisEvent ) 
{
    return ES_PostToService( MyPriority, ThisEvent);
}

/****************************************************************************
 Function
     RunBlink

 Parameters
     ES_Event: the event to process

 Returns
     ES_Event: an event to return

 Description
     the run function for the top level state machine 
 Notes
     uses nested switch/case to implement the machine.
 Author
     Michael Lin
****************************************************************************/
ES_Event RunBlink( ES_Event CurrentEvent )
{
    ES_Event ReturnEvent;

    ReturnEvent.EventType = ES_NO_EVENT; // assume no errors 
    switch ( CurrentState )
    {
        case InitializeBlink:
            CurrentState = Blinking;
            ES_Timer_InitTimer(BlinkTimer, 1);
            break;
        case Blinking:
            if ( (CurrentEvent.EventType == ES_TIMEOUT) && (CurrentEvent.EventParam == BlinkTimer) ) {
                LATA2 ^= 1;
                ES_Timer_InitTimer(BlinkTimer, 1);
            }
            break;
    }
    return ReturnEvent;
}