/*----------------------------- Include Files -----------------------------*/
/* include header files for the framework and this service
*/
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "PIC16F1788.h"
#include "ButtonDebounce.h"
#include "CommService.h"

// Data Private to Module
static uint8_t LastButtonState;
static uint8_t MyPriority;
static ButtonState_t CurrentState;

// Function Definitions
bool InitButtonDB (uint8_t Priority){
	MyPriority = Priority;
	
    //Use RB0
	TRISB0  = 1;              // make RB0 input	
	
	// Sample the button port pin
	LastButtonState = RB0;
	// Set CurrentState to be Debouncing
	CurrentState = Debouncing;
	// Start debounce timer
	ES_Timer_InitTimer(DBTimer, 30); // Timer0, debounce time of 30 ms
	return true;
}

bool PostButton( ES_Event ThisEvent ){
	return ES_PostToService( MyPriority, ThisEvent);
}

bool CheckButtonEvents(void){
	// Local Variables
	ES_Event ThisEvent;
	uint8_t ReturnVal = false;
	// Set CurrentButtonState to state read from port pin
	uint8_t CurrentButtonState = RB0;
	// If the CurrentButtonState is different from the LastButtonState
	if (CurrentButtonState != LastButtonState){
		ReturnVal = true;
		// If the CurrentButtonState is down
		if (CurrentButtonState == 1) {
			ThisEvent.EventType = ES_BUTTON_DOWN;
			PostButton(ThisEvent);
		}
		else{ // button is up
			ThisEvent.EventType = ES_BUTTON_UP;
			PostButton(ThisEvent);
		}
	}
	LastButtonState = CurrentButtonState;
	return ReturnVal;
}

ES_Event RunButtonDB( ES_Event ThisEvent ){
	// If CurrentState is Debouncing
	if (CurrentState == Debouncing){
		// If EventType is ES_TIMEOUT & parameter is debounce timer number
		if ( (ThisEvent.EventType == ES_TIMEOUT) && (ThisEvent.EventParam == DBTimer) ){ // 0 is debounce timer number
			CurrentState = Ready2Sample;
		}
	}
	// Else if CurrentState is Ready2Sample
	else if (CurrentState == Ready2Sample){
		// If EventType is ButtonUp
		if (ThisEvent.EventType == ES_BUTTON_UP){
			// Start debounce timer
			ES_Timer_InitTimer(DBTimer,30);
			CurrentState = Debouncing;
            ES_Event ThisEvent;
            ThisEvent.EventType = ES_Transmit;
            PostCommService( ThisEvent );
			// Post DB_button_up event?
		}
		// If EventType is ButtonDown
		else if (ThisEvent.EventType == ES_BUTTON_DOWN){
			// Start debounce timer
			ES_Timer_InitTimer(DBTimer,30);
			CurrentState = Debouncing;
			// Post DBButtonDown
		}
	}
	ThisEvent.EventType = ES_NO_EVENT;
	return ThisEvent;
}
