#include <htc.h>
#include "ES_Port.h"

static unsigned char TickCount;


/****************************************************************************
 Function
     ES_Timer_Init
 Parameters
     unsigned char Rate set to one of the TMR_RATE_XX values to set the
     RTI rate
 Returns
     None.
 Description
     Initializes the timer module by setting up the RTI with the requested
    rate
 Notes
     For PIC testing: a quick hack that fixes the rate at 10ms
 Author
     J. Edward Carryer, 02/24/97 14:23
****************************************************************************/
void ES_Timer_Init(TimerRate_t Rate)
{
   // keep the compiler from compaining about not referenceing param
   // since in this simple test, we are not allowing a different rate
   Rate;     
   /*  Timer 1 interrupt disabled.
    *  CCP Module 1 interrupt enabled.
    */
   PIE1    = _PIE1_CCP1IE_MASK;
   /*
    *  Peripheral interrupts enabled
    *  Global interrupt disabled during initialization
    */
   INTCON  = _INTCON_PEIE_MASK;
    /***** CCP Module 1 Code ****
    *  CCP Module 1 enabled
    *  CCP Module 1 in Compare mode
    *  Software interrupt on match
    */
   CCP1CON = ( _CCP1CON_CCP1M1_MASK | _CCP1CON_CCP1M3_MASK );
   /*
    *  Comparator value set to 1
    */
   CCPR1L  = 0b00000001;
   /*
    *  Comparator value set to 1
    */
   CCPR1H  = 0b00000000;
   
   /***** Timer 1 Code ****
    *  Timer is active and running
    *  Timer derived from system clock
    *  Time is not Gated
    *  Timer prescaler is 1:4
    */
   T1CON = ( _T1CON_T1CKPS1_MASK | _T1CON_T1CKPS0_MASK | _T1CON_TMR1ON_MASK );
   EnableInterrupts;                /* make sure that interrupts are enabled*/
}

void IncrCounter( void )
{
    TickCount++;   
}

unsigned char IsTimerPending( void )
{
   if (TickCount > 0)
   {
      TickCount--;
      return (1);
   }else
      return (0);
}