/****************************************************************************
 Module
   d:\me218\DevProjects\EventFramrwork\RemoteLockSM.c

 Revision
   1.0.1

 Description
   This is the implementation of the Remote Lock example state machine

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 10/23/11 21:39 jec      began conversion from EF_SMTemplate.c 
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "ES_Timers.h"
#include "ES_Queue.h"
#include "RemoteLockSM.h"
#include <stdio.h>

/*----------------------------- Module Defines ----------------------------*/
#define QUEUE_SIZE 3

// This define assumes a 10mS tick rate
#define TWO_SEC 200

/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine.They should be functions
   relevant to the behavior of this state machine
*/

/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well.
// type of state variable should match that of enum in header file
static RemoteLockState_t CurrentState;

// everybody needs a local store for the service's priority too!
static uint8_t MyPriority;


/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     InitRemoteLockSM
 Parameters
     uint8_t Priority
 Returns
     boolean, False if error in initialization, True otherwise
 Description
     Saves away the priority assigned to this service and posts an ES_INT
     event to this state machine
 Notes

 Author
     J. Edward Carryer, 01/26/12, 17:42
****************************************************************************/
bool InitRemoteLockSM ( uint8_t Priority )
{
  ES_Event ThisEvent;

  MyPriority = Priority;
  // put us into the Initial PseudoState
  CurrentState = InitPState;
  // post the initial transition event
  ThisEvent.EventType = ES_INIT;
  if (ES_PostToService( MyPriority, ThisEvent) == true)
  {
      return true;
  }else
  {
      return false;
  }
}

/****************************************************************************
 Function
     PostRemoteLockSM
 Parameters
     EF_Event ThisEvent , the event to post to the queue
 Returns
     boolean False if the Enqueue operation failed, True otherwise
 Description
     Posts an event to the state machine's queue
 Notes

 Author
     J. Edward Carryer, 01/26/12, 21:45
****************************************************************************/
bool PostRemoteLockSM( ES_Event ThisEvent )
{
  return ES_PostToService( MyPriority, ThisEvent);

}

/****************************************************************************
 Function
    RunRemoteLockSM
 Parameters
   ES_Event : the event to process
 Returns
   ES_Event, ES_NO_EVENT if no error ES_ERROR otherwise
 Description
   implements a state machine for the remote lock example problem
 Notes
   uses nested switch/case to implement the machine.
 Author
   J. Edward Carryer, 01/26/12, 17:46
****************************************************************************/
ES_Event RunRemoteLockSM( ES_Event CurrentEvent )
{
  ES_Event ReturnEvent;
  
  ReturnEvent.EventType = ES_NO_EVENT; // assume no problems will occur
  
  switch ( CurrentState )
  {
      case InitPState :       // If current state is initial Psedudo State
          if ( CurrentEvent.EventType == ES_INIT )// only respond to EF_Init
          {
              // this is where you would put any actions associated with the
              // transition from the initial pseudo-state into the actual
              // initial state
              //puts("All Unlocked Init\r");
              // now put the machine into the actual initial state
              CurrentState = UnlockWaiting;
           }
           break;

      case UnlockWaiting :       
          switch ( CurrentEvent.EventType )
          {
              case ES_UNLOCK : 
              {
                  // Execute action functions for UnlockWaiting : ES_UNLOCK
                  //puts("Unlock Driver \r");
                  ES_Timer_InitTimer(0, TWO_SEC );
                  CurrentState = _1UnlockPress;
               }
               break;

              case ES_LOCK : 
              {
                  // Execute action functions for UnlockWaiting : ES_LOCK
                  //puts("Lock All \r");
                  CurrentState = Locked;
               }
               break;

          }                           // end switch on CurrentEvent
          break;

      case _1UnlockPress :       // If current state is 1UnlockPress
          switch ( CurrentEvent.EventType )
          {
              case ES_UNLOCK : 
              {
                  // Execute action function for 1UnlockPress : ES_UNLOCK
                  //puts("Unlock Other \r");
                  CurrentState = _2UnlockPresses;//Decide what the next state will be
               }
               break;

              case ES_LOCK : 
              {
                  // Execute action function for 1UnlockPress : ES_LOCK
                  //puts("Lock All \r");
                  CurrentState = Locked;//Decide what the next state will be
               }
               break;

              case ES_TIMEOUT : 
              {
                  // Execute action function for 1UnlockPress : ES_TIMEOUT
                  //puts("Timeout \r");
                  CurrentState = UnlockWaiting;//Decide what the next state will be
               }
               break;

          }                           // end switch on CurrentEvent
          break;

      case _2UnlockPresses :       // If current state is 1UnlockPress
          switch ( CurrentEvent.EventType )
          {
              case ES_UNLOCK : 
              {
                  // Execute action function for 2UnlockPress : ES_UNLOCK
                  //puts("Beep \r");
                  CurrentState = UnlockWaiting;//Decide what the next state will be
               }
               break;

              case ES_LOCK : 
              {
                  // Execute action function for 2UnlockPress : ES_LOCK
                  //puts("Lock All \r");
                  CurrentState = Locked;//Decide what the next state will be
               }
               break;

              case ES_TIMEOUT : 
              {
                  // Execute action function for 2UnlockPress : ES_TIMEOUT
                  //puts("Timeout \r");
                  CurrentState = UnlockWaiting;//Decide what the next state will be
               }
               break;

          }                           // end switch on CurrentEvent
          break;

      case Locked :       
          switch ( CurrentEvent.EventType )
          {
              case ES_UNLOCK : 
              {
                  // Execute action functions for UnlockWaiting : ES_UNLOCK
                  //puts("Unlock Driver \r");
                  ES_Timer_InitTimer(0, TWO_SEC );
                  CurrentState = _1UnlockPress;
               }
               break;

              case ES_LOCK : 
              {
                  // Execute action functions for UnlockWaiting : ES_LOCK
                  //puts("Beep \r");
               }
               break;

          }                           // end switch on CurrentEvent
          break;


  }                                   // end switch on Current State
  return ReturnEvent;
}


/****************************************************************************
 Function
     QueryRemoteLockSM
 Parameters
     None
 Returns
     RemoteLockState_t The current state of the state machine
 Description
     returns the current state of the state machine
 Notes

 Author
     J. Edward Carryer, 10/23/11, 22:27
****************************************************************************/
RemoteLockState_t QueryRemoteLockSM ( void )
{
   return(CurrentState);
}

/***************************************************************************
 private functions
 ***************************************************************************/

