/*
 * File:   main.c
 * Author: mlinyang
 *
 * Created on May 5, 2016, 12:50 PM
 */

#include <xc.h>
#include <stdio.h>
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "ES_Timers.h"
#include "ES_Port.h"
#include "CommService.h"
#include "PairingSM.h"
#include "PIC16F1788.h"
#include "pic.h"

// Config 1
#pragma config FOSC = INTOSC
#pragma config WDTE = OFF
#pragma config PWRTE = ON
#pragma config MCLRE = OFF
#pragma config BOREN = OFF

// Config 2
#pragma config PLLEN = ON

void main(void) {
    ES_Return_t ErrorType;
    OSCCON  = 0b01111000; // Configure PIC16F1788 to 8 MHz
    
    // Init the GPIO hardware Port A
    PORTA  = 0b00000000;      // first clear PORTA bits
    LATA   = 0b00000000;      // then clear LATA bits
    ANSELA = 0b00000000;      // turn pins into digital
    TRISA  = 0b00000000;      // make RA0 output
    // Init the GPIO hardware Port B
    PORTB  = 0b00000000;      // first clear PORTB bits
    LATB   = 0b00000000;      // then clear LATB bits
    ANSELB = 0b00001000;      // turn pins into digital
    
    ErrorType = ES_Initialize(ES_Timer_RATE_1mS);
    if ( ErrorType == Success ) {
        ErrorType = ES_Run();
    }
    for(;;)
    ;
}

// timing parameter. This is based on an 8MHz internal clock and a divide by
// 4 prescaler on Timer 1 for the CCP and a desired tick rate of 100Hz.
#define COUNTS_PER_TICK (((8000000L/4)/4)/500)

void interrupt ISR ( void )
{
   // make these static for speed in ISR
   static unsigned int LastTime;
   static unsigned int NextTime;
   static unsigned int ADCRead;

	if ( CCP1IF == 1 ) // this was a timer interrupt
   {
      CCP1IF = 0; // clear source flag
      LastTime = ((unsigned int)CCPR1H << 8)+ CCPR1L;
      NextTime = LastTime + COUNTS_PER_TICK;
      CCPR1H = (unsigned char)(NextTime >> 8);
      CCPR1L = (unsigned char)(NextTime & 0xFF);
	  IncrCounter(); // count number of timer tick that need to be processed
   } else if ( RCIF == 1 )
   {
      RCIF = 0; // clear receive interrupt flag
      uint8_t RXData;
      RXData = RCREG;
      ES_Event ThisEvent;
      ThisEvent.EventType = ES_ReceivedByte;
      ThisEvent.EventParam = RXData;
      PostCommService( ThisEvent );
   } else if (ADIF == 1) // this was an ADC interrupt, indicating the end of a conversion
   {
       //LATA4^=1;
       ADIF = 0; // clear ADC interrupt flag
       ADCRead = ADRESH; //read the high 8 bytes of the result of the conversion
       //ADRESH = 0x00;
       ES_Event ThisEvent;
       ThisEvent.EventType = ES_ADCNewRead;
       ThisEvent.EventParam = ADCRead;
       PostPairingSM( ThisEvent );
   }
// add your interrupt processing here
	   
}